@@ -1,12 +1,14 @@
 /* tsprintf.c -- test file for mpfr_sprintf, mpfr_vsprintf, mpfr_snprintf,
    and mpfr_vsnprintf
 
-Copyright 2007, 2008, 2009 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+Contributed by the AriC and Caramel projects, INRIA.
+
+This file is part of the GNU MPFR Library.
 
 The GNU MPFR Library is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
-the Free Software Foundation; either version 2.1 of the License, or (at your
+the Free Software Foundation; either version 3 of the License, or (at your
 option) any later version.
 
 The GNU MPFR Library is distributed in the hope that it will be useful, but
@@ -15,21 +17,14 @@ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
 License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
-along with the GNU MPFR Library; see the file COPYING.LIB.  If not, write to
-the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
-MA 02110-1301, USA. */
+along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
+http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #ifdef HAVE_STDARG
 #include <stdarg.h>
 
-#if defined (__cplusplus)
-#include <cstdio>
-#else
-#include <stdio.h>
-#endif
-
 #include <stdlib.h>
-#include <string.h>
 #include <float.h>
 
 #ifdef HAVE_LOCALE_H
@@ -207,6 +202,9 @@ native_types (void)
   sprintf (buf, "%s", s);
   check_vsprintf (buf, "%s", s);
 
+  sprintf (buf, "--%s++", "");
+  check_vsprintf (buf, "--%s++", "");
+
   sprintf (buf, "%u", ui);
   check_vsprintf (buf, "%u", ui);
 
@@ -225,43 +223,64 @@ decimal (void)
 
   /* specifier 'P' for precision */
   check_vsprintf ("128", "%Pu", p);
+  check_vsprintf ("00128", "%.5Pu", p);
 
   /* special numbers */
   mpfr_set_inf (x, 1);
   check_sprintf (pinf_str, "%Re", x);
+  check_sprintf (pinf_str, "%RUe", x);
   check_sprintf (pinf_uc_str, "%RE", x);
+  check_sprintf (pinf_uc_str, "%RDE", x);
   check_sprintf (pinf_str, "%Rf", x);
+  check_sprintf (pinf_str, "%RYf", x);
   check_sprintf (pinf_uc_str, "%RF", x);
+  check_sprintf (pinf_uc_str, "%RZF", x);
   check_sprintf (pinf_str, "%Rg", x);
+  check_sprintf (pinf_str, "%RNg", x);
   check_sprintf (pinf_uc_str, "%RG", x);
+  check_sprintf (pinf_uc_str, "%RUG", x);
   check_sprintf ("       inf", "%010Re", x);
+  check_sprintf ("       inf", "%010RDe", x);
 
   mpfr_set_inf (x, -1);
   check_sprintf (minf_str, "%Re", x);
+  check_sprintf (minf_str, "%RYe", x);
   check_sprintf (minf_uc_str, "%RE", x);
+  check_sprintf (minf_uc_str, "%RZE", x);
   check_sprintf (minf_str, "%Rf", x);
+  check_sprintf (minf_str, "%RNf", x);
   check_sprintf (minf_uc_str, "%RF", x);
+  check_sprintf (minf_uc_str, "%RUF", x);
   check_sprintf (minf_str, "%Rg", x);
+  check_sprintf (minf_str, "%RDg", x);
   check_sprintf (minf_uc_str, "%RG", x);
+  check_sprintf (minf_uc_str, "%RYG", x);
   check_sprintf ("      -inf", "%010Re", x);
+  check_sprintf ("      -inf", "%010RZe", x);
 
   mpfr_set_nan (x);
   check_sprintf (nan_str, "%Re", x);
+  check_sprintf (nan_str, "%RNe", x);
   check_sprintf (nan_uc_str, "%RE", x);
+  check_sprintf (nan_uc_str, "%RUE", x);
   check_sprintf (nan_str, "%Rf", x);
+  check_sprintf (nan_str, "%RDf", x);
   check_sprintf (nan_uc_str, "%RF", x);
+  check_sprintf (nan_uc_str, "%RYF", x);
   check_sprintf (nan_str, "%Rg", x);
+  check_sprintf (nan_str, "%RZg", x);
   check_sprintf (nan_uc_str, "%RG", x);
+  check_sprintf (nan_uc_str, "%RNG", x);
   check_sprintf ("       nan", "%010Re", x);
 
   /* positive numbers */
-  mpfr_set_str (x, "18993474.61279296875", 10, GMP_RNDN);
-  mpfr_set_ui (z, 0, GMP_RNDD);
+  mpfr_set_str (x, "18993474.61279296875", 10, MPFR_RNDN);
+  mpfr_set_ui (z, 0, MPFR_RNDD);
 
   /* simplest case right justified */
   check_sprintf ("      1.899347461279296875e+07", "%30Re", x);
   check_sprintf ("                         2e+07", "%30.0Re", x);
-  check_sprintf ("          18993474.61279296875", "%30Rf", x);
+  check_sprintf ("               18993474.612793", "%30Rf", x);
   check_sprintf ("              18993474.6127930", "%30.7Rf", x);
   check_sprintf ("                   1.89935e+07", "%30Rg", x);
   check_sprintf ("                         2e+07", "%30.0Rg", x);
@@ -283,12 +302,12 @@ decimal (void)
   check_sprintf ("+0e+00                        ", "%+-30.0Re", z);
   check_sprintf ("+0                            ", "%+-30.0Rf", z);
   /* decimal point, left justified, precision and rounding parameter */
-  check_vsprintf ("1.9E+07   ", "%#-10.*R*E", 1, GMP_RNDN, x);
-  check_vsprintf ("2.E+07    ", "%#*.*R*E", -10, 0, GMP_RNDN, x);
-  check_vsprintf ("2.E+07    ", "%#-10.*R*G", 0, GMP_RNDN, x);
-  check_vsprintf ("0.E+00    ", "%#-10.*R*E", 0, GMP_RNDN, z);
-  check_vsprintf ("0.        ", "%#-10.*R*F", 0, GMP_RNDN, z);
-  check_vsprintf ("0.        ", "%#-10.*R*G", 0, GMP_RNDN, z);
+  check_vsprintf ("1.9E+07   ", "%#-10.*R*E", 1, MPFR_RNDN, x);
+  check_vsprintf ("2.E+07    ", "%#*.*R*E", -10, 0, MPFR_RNDN, x);
+  check_vsprintf ("2.E+07    ", "%#-10.*R*G", 0, MPFR_RNDN, x);
+  check_vsprintf ("0.E+00    ", "%#-10.*R*E", 0, MPFR_RNDN, z);
+  check_vsprintf ("0.        ", "%#-10.*R*F", 0, MPFR_RNDN, z);
+  check_vsprintf ("0.        ", "%#-10.*R*G", 0, MPFR_RNDN, z);
   /* sign or space */
   check_sprintf (" 1.899e+07", "% .3RNe", x);
   check_sprintf (" 2e+07",     "% .0RNe", x);
@@ -305,8 +324,8 @@ decimal (void)
   check_sprintf (" 1.E+07    ", "%- #11.0RDE", x);
 
   /* negative numbers */
-  mpfr_mul_si (x, x, -1, GMP_RNDD);
-  mpfr_mul_si (z, z, -1, GMP_RNDD);
+  mpfr_mul_si (x, x, -1, MPFR_RNDD);
+  mpfr_mul_si (z, z, -1, MPFR_RNDD);
 
   /* sign + or - */
   check_sprintf ("  -1.8e+07", "%+10.1RUe", x);
@@ -316,53 +335,116 @@ decimal (void)
 
 
   /* neighborhood of 1 */
-  mpfr_set_str (x, "0.9999", 10, GMP_RNDN);
-  check_sprintf ("1E+00     ", "%-10.0RE", x);
-  check_sprintf ("1.0E+00   ", "%-10.1RE", x);
-  check_sprintf ("9.9990E-01", "%-10.4RE", x);
-  check_sprintf ("1.0       ", "%-10.1RF", x);
-  check_sprintf ("0.9999    ", "%-10.4RF", x);
-  check_sprintf ("1         ", "%-10.0RG", x);
-  check_sprintf ("1         ", "%-10.1RG", x);
-  check_sprintf ("0.9999    ", "%-10.4RG", x);
-  check_sprintf ("1.        ", "%-#10.0RG", x);
-  check_sprintf ("1.        ", "%-#10.1RG", x);
-  check_sprintf ("1.0       ", "%-#10.2RG", x);
-  check_sprintf ("0.9999    ", "%-#10.4RG", x);
+  mpfr_set_str (x, "0.99993896484375", 10, MPFR_RNDN);
+  check_sprintf ("9.9993896484375E-01 ", "%-20RE", x);
+  check_sprintf ("9.9993896484375E-01 ", "%-20.RE", x);
+  check_sprintf ("1E+00               ", "%-20.0RE", x);
+  check_sprintf ("1.0E+00             ", "%-20.1RE", x);
+  check_sprintf ("1.00E+00            ", "%-20.2RE", x);
+  check_sprintf ("9.999E-01           ", "%-20.3RE", x);
+  check_sprintf ("9.9994E-01          ", "%-20.4RE", x);
+  check_sprintf ("0.999939            ", "%-20RF", x);
+  check_sprintf ("0.999939            ", "%-20.RF", x);
+  check_sprintf ("1                   ", "%-20.0RF", x);
+  check_sprintf ("1.0                 ", "%-20.1RF", x);
+  check_sprintf ("1.00                ", "%-20.2RF", x);
+  check_sprintf ("1.000               ", "%-20.3RF", x);
+  check_sprintf ("0.9999              ", "%-20.4RF", x);
+  check_sprintf ("0.999939            ", "%-#20RF", x);
+  check_sprintf ("0.999939            ", "%-#20.RF", x);
+  check_sprintf ("1.                  ", "%-#20.0RF", x);
+  check_sprintf ("1.0                 ", "%-#20.1RF", x);
+  check_sprintf ("1.00                ", "%-#20.2RF", x);
+  check_sprintf ("1.000               ", "%-#20.3RF", x);
+  check_sprintf ("0.9999              ", "%-#20.4RF", x);
+  check_sprintf ("1                   ", "%-20.0RG", x);
+  check_sprintf ("1                   ", "%-20.1RG", x);
+  check_sprintf ("1                   ", "%-20.2RG", x);
+  check_sprintf ("1                   ", "%-20.3RG", x);
+  check_sprintf ("0.9999              ", "%-20.4RG", x);
+  check_sprintf ("0.999939            ", "%-#20RG", x);
+  check_sprintf ("0.999939            ", "%-#20.RG", x);
+  check_sprintf ("1.                  ", "%-#20.0RG", x);
+  check_sprintf ("1.                  ", "%-#20.1RG", x);
+  check_sprintf ("1.0                 ", "%-#20.2RG", x);
+  check_sprintf ("1.00                ", "%-#20.3RG", x);
+  check_sprintf ("0.9999              ", "%-#20.4RG", x);
 
   /* multiple of 10 */
-  mpfr_set_str (x, "1e17", 10, GMP_RNDN);
+  mpfr_set_str (x, "1e17", 10, MPFR_RNDN);
   check_sprintf ("1e+17", "%Re", x);
   check_sprintf ("1.000e+17", "%.3Re", x);
-  check_sprintf ("100000000000000000", "%Rf", x);
+  check_sprintf ("100000000000000000", "%.0Rf", x);
   check_sprintf ("100000000000000000.0", "%.1Rf", x);
-  check_sprintf ("100000000000000000", "%'Rf", x);
+  check_sprintf ("100000000000000000.000000", "%'Rf", x);
   check_sprintf ("100000000000000000.0", "%'.1Rf", x);
 
-  mpfr_ui_div (x, 1, x, GMP_RNDN); /* x=1e-17 */
+  mpfr_ui_div (x, 1, x, MPFR_RNDN); /* x=1e-17 */
   check_sprintf ("1e-17", "%Re", x);
-  check_sprintf ("0.00000000000000001", "%Rf", x);
+  check_sprintf ("0.000000", "%Rf", x);
   check_sprintf ("1e-17", "%Rg", x);
   check_sprintf ("0.0", "%.1RDf", x);
+  check_sprintf ("0.0", "%.1RZf", x);
   check_sprintf ("0.1", "%.1RUf", x);
+  check_sprintf ("0.1", "%.1RYf", x);
   check_sprintf ("0", "%.0RDf", x);
+  check_sprintf ("0", "%.0RZf", x);
   check_sprintf ("1", "%.0RUf", x);
+  check_sprintf ("1", "%.0RYf", x);
+
+  /* multiple of 10 with 'g' style */
+  mpfr_set_str (x, "10", 10, MPFR_RNDN);
+  check_sprintf ("10", "%Rg", x);
+  check_sprintf ("1e+01", "%.0Rg", x);
+  check_sprintf ("1e+01", "%.1Rg", x);
+  check_sprintf ("10", "%.2Rg", x);
+
+  mpfr_ui_div (x, 1, x, MPFR_RNDN);
+  check_sprintf ("0.1", "%Rg", x);
+  check_sprintf ("0.1", "%.0Rg", x);
+  check_sprintf ("0.1", "%.1Rg", x);
+
+  mpfr_set_str (x, "1000", 10, MPFR_RNDN);
+  check_sprintf ("1000", "%Rg", x);
+  check_sprintf ("1e+03", "%.0Rg", x);
+  check_sprintf ("1e+03", "%.3Rg", x);
+  check_sprintf ("1000", "%.4Rg", x);
+
+  mpfr_ui_div (x, 1, x, MPFR_RNDN);
+  check_sprintf ("0.001", "%Rg", x);
+  check_sprintf ("0.001", "%.0Rg", x);
+  check_sprintf ("0.001", "%.1Rg", x);
+
+  mpfr_set_str (x, "100000", 10, MPFR_RNDN);
+  check_sprintf ("100000", "%Rg", x);
+  check_sprintf ("1e+05", "%.0Rg", x);
+  check_sprintf ("1e+05", "%.5Rg", x);
+  check_sprintf ("100000", "%.6Rg", x);
+
+  mpfr_ui_div (x, 1, x, MPFR_RNDN);
+  check_sprintf ("1e-05", "%Rg", x);
+  check_sprintf ("1e-05", "%.0Rg", x);
+  check_sprintf ("1e-05", "%.1Rg", x);
 
   /* check rounding mode */
-  mpfr_set_str (x, "0.0076", 10, GMP_RNDN);
+  mpfr_set_str (x, "0.0076", 10, MPFR_RNDN);
   check_sprintf ("0.007", "%.3RDF", x);
   check_sprintf ("0.007", "%.3RZF", x);
   check_sprintf ("0.008", "%.3RF", x);
   check_sprintf ("0.008", "%.3RUF", x);
+  check_sprintf ("0.008", "%.3RYF", x);
+  check_vsprintf ("0.008", "%.3R*F", MPFR_RNDA, x);
 
-  /* limit test for the choice beetwen %f-style and %g-style */
-  mpfr_set_str (x, "0.0000999", 10, GMP_RNDN);
-  check_sprintf ("0.0001", "%.0Rg", x);
-  check_sprintf ("9e-05", "%.0RDg", x);
-  check_sprintf ("0.0001", "%.2Rg", x);
+  /* check limit between %f-style and %g-style */
+  mpfr_set_str (x, "0.0000999", 10, MPFR_RNDN);
+  check_sprintf ("0.0001",   "%.0Rg", x);
+  check_sprintf ("9e-05",    "%.0RDg", x);
+  check_sprintf ("0.0001",   "%.1Rg", x);
+  check_sprintf ("0.0001",   "%.2Rg", x);
+  check_sprintf ("9.99e-05", "%.3Rg", x);
 
   /* trailing zeros */
-  mpfr_set_si_2exp (x, -1, -15, GMP_RNDN); /* x=-2^-15 */
+  mpfr_set_si_2exp (x, -1, -15, MPFR_RNDN); /* x=-2^-15 */
   check_sprintf ("-3.0517578125e-05", "%.30Rg", x);
   check_sprintf ("-3.051757812500000000000000000000e-05", "%.30Re", x);
   check_sprintf ("-3.05175781250000000000000000000e-05", "%#.30Rg", x);
@@ -370,13 +452,13 @@ decimal (void)
 
   /* bug 20081023 */
   check_sprintf ("-3.0517578125e-05", "%.30Rg", x);
-  mpfr_set_str (x, "1.9999", 10, GMP_RNDN);
+  mpfr_set_str (x, "1.9999", 10, MPFR_RNDN);
   check_sprintf ("1.999900  ", "%-#10.7RG", x);
   check_sprintf ("1.9999    ", "%-10.7RG", x);
-  mpfr_set_ui (x, 1, GMP_RNDN);
+  mpfr_set_ui (x, 1, MPFR_RNDN);
   check_sprintf ("1.00000000000000000000000000000", "%#.30Rg", x);
   check_sprintf ("1", "%.30Rg", x);
-  mpfr_set_ui (x, 0, GMP_RNDN);
+  mpfr_set_ui (x, 0, MPFR_RNDN);
   check_sprintf ("0.000000000000000000000000000000", "%#.30Rg", x);
   check_sprintf ("0", "%.30Rg", x);
 
@@ -384,13 +466,89 @@ decimal (void)
   mpfr_set_prec (x, 53);
 
   /* Exponent zero has a plus sign */
-  mpfr_set_str (x, "-9.95645044213728791504536275169812142849e-01", 10, GMP_RNDN);
+  mpfr_set_str (x, "-9.95645044213728791504536275169812142849e-01", 10,
+                MPFR_RNDN);
   check_sprintf ("-1.0e+00", "%- #0.1Re", x);
 
   /* Decimal point and no figure after it with '#' flag and 'G' style */
-  mpfr_set_str (x, "-9.90597761233942053494e-01", 10, GMP_RNDN);
+  mpfr_set_str (x, "-9.90597761233942053494e-01", 10, MPFR_RNDN);
   check_sprintf ("-1.", "%- #0.1RG", x);
 
+  /* precision zero */
+  mpfr_set_d (x, 9.5, MPFR_RNDN);
+  check_sprintf ("9",    "%.0RDf", x);
+  check_sprintf ("10",    "%.0RUf", x);
+
+  mpfr_set_d (x, 19.5, MPFR_RNDN);
+  check_sprintf ("19",    "%.0RDf", x);
+  check_sprintf ("20",    "%.0RUf", x);
+
+  mpfr_set_d (x, 99.5, MPFR_RNDN);
+  check_sprintf ("99",    "%.0RDf", x);
+  check_sprintf ("100",   "%.0RUf", x);
+
+  mpfr_set_d (x, -9.5, MPFR_RNDN);
+  check_sprintf ("-10",    "%.0RDf", x);
+  check_sprintf ("-10",    "%.0RYf", x);
+  check_sprintf ("-10",    "%.0Rf", x);
+  check_sprintf ("-1e+01", "%.0Re", x);
+  check_sprintf ("-1e+01", "%.0Rg", x);
+  mpfr_set_ui_2exp (x, 1, -1, MPFR_RNDN);
+  check_sprintf ("0",      "%.0Rf", x);
+  check_sprintf ("5e-01",  "%.0Re", x);
+  check_sprintf ("0.5",    "%.0Rg", x);
+  mpfr_set_ui_2exp (x, 3, -1, MPFR_RNDN);
+  check_sprintf ("2",      "%.0Rf", x);
+  mpfr_set_ui_2exp (x, 5, -1, MPFR_RNDN);
+  check_sprintf ("2",      "%.0Rf", x);
+  mpfr_set_ui (x, 0x1f, MPFR_RNDN);
+  check_sprintf ("0x1p+5", "%.0Ra", x);
+  mpfr_set_ui (x, 3, MPFR_RNDN);
+  check_sprintf ("1p+2",   "%.0Rb", x);
+
+  /* round to next ten power with %f but not with %g */
+  mpfr_set_str (x, "-6.64464380544039223686e-02", 10, MPFR_RNDN);
+  check_sprintf ("-0.1",  "%.1Rf", x);
+  check_sprintf ("-0.0",  "%.1RZf", x);
+  check_sprintf ("-0.07", "%.1Rg", x);
+  check_sprintf ("-0.06", "%.1RZg", x);
+
+  /* round to next ten power and do not remove trailing zeros */
+  mpfr_set_str (x, "9.98429393291486722006e-02", 10, MPFR_RNDN);
+  check_sprintf ("0.1",   "%#.1Rg", x);
+  check_sprintf ("0.10",  "%#.2Rg", x);
+  check_sprintf ("0.099", "%#.2RZg", x);
+
+  /* Halfway cases */
+  mpfr_set_str (x, "1.5", 10, MPFR_RNDN);
+  check_sprintf ("2e+00", "%.0Re", x);
+  mpfr_set_str (x, "2.5", 10, MPFR_RNDN);
+  check_sprintf ("2e+00", "%.0Re", x);
+  mpfr_set_str (x, "9.5", 10, MPFR_RNDN);
+  check_sprintf ("1e+01", "%.0Re", x);
+  mpfr_set_str (x, "1.25", 10, MPFR_RNDN);
+  check_sprintf ("1.2e+00", "%.1Re", x);
+  mpfr_set_str (x, "1.75", 10, MPFR_RNDN);
+  check_sprintf ("1.8e+00", "%.1Re", x);
+  mpfr_set_str (x, "-0.5", 10, MPFR_RNDN);
+  check_sprintf ("-0", "%.0Rf", x);
+  mpfr_set_str (x, "1.25", 10, MPFR_RNDN);
+  check_sprintf ("1.2", "%.1Rf", x);
+  mpfr_set_str (x, "1.75", 10, MPFR_RNDN);
+  check_sprintf ("1.8", "%.1Rf", x);
+  mpfr_set_str (x, "1.5", 10, MPFR_RNDN);
+  check_sprintf ("2", "%.1Rg", x);
+  mpfr_set_str (x, "2.5", 10, MPFR_RNDN);
+  check_sprintf ("2", "%.1Rg", x);
+  mpfr_set_str (x, "9.25", 10, MPFR_RNDN);
+  check_sprintf ("9.2", "%.2Rg", x);
+  mpfr_set_str (x, "9.75", 10, MPFR_RNDN);
+  check_sprintf ("9.8", "%.2Rg", x);
+
+  /* assertion failure in r6320 */
+  mpfr_set_str (x, "-9.996", 10, MPFR_RNDN);
+  check_sprintf ("-10.0", "%.1Rf", x);
+
   mpfr_clears (x, z, (mpfr_ptr) 0);
   return 0;
 }
@@ -404,39 +562,54 @@ hexadecimal (void)
   /* special */
   mpfr_set_inf (x, 1);
   check_sprintf (pinf_str, "%Ra", x);
+  check_sprintf (pinf_str, "%RUa", x);
+  check_sprintf (pinf_str, "%RDa", x);
   check_sprintf (pinf_uc_str, "%RA", x);
+  check_sprintf (pinf_uc_str, "%RYA", x);
+  check_sprintf (pinf_uc_str, "%RZA", x);
+  check_sprintf (pinf_uc_str, "%RNA", x);
 
   mpfr_set_inf (x, -1);
   check_sprintf (minf_str, "%Ra", x);
+  check_sprintf (minf_str, "%RYa", x);
+  check_sprintf (minf_str, "%RZa", x);
+  check_sprintf (minf_str, "%RNa", x);
   check_sprintf (minf_uc_str, "%RA", x);
+  check_sprintf (minf_uc_str, "%RUA", x);
+  check_sprintf (minf_uc_str, "%RDA", x);
 
   mpfr_set_nan (x);
   check_sprintf (nan_str, "%Ra", x);
   check_sprintf (nan_uc_str, "%RA", x);
 
   /* regular numbers */
-  mpfr_set_str (x, "FEDCBA9.87654321", 16, GMP_RNDN);
-  mpfr_set_ui (z, 0, GMP_RNDZ);
+  mpfr_set_str (x, "FEDCBA9.87654321", 16, MPFR_RNDN);
+  mpfr_set_ui (z, 0, MPFR_RNDZ);
 
   /* simplest case right justified */
   check_sprintf ("   0xf.edcba987654321p+24", "%25Ra", x);
-  check_sprintf ("                  0x8p+25", "%25.0Ra", x);
+  check_sprintf ("   0xf.edcba987654321p+24", "%25RUa", x);
+  check_sprintf ("   0xf.edcba987654321p+24", "%25RDa", x);
+  check_sprintf ("   0xf.edcba987654321p+24", "%25RYa", x);
+  check_sprintf ("   0xf.edcba987654321p+24", "%25RZa", x);
+  check_sprintf ("   0xf.edcba987654321p+24", "%25RNa", x);
+  check_sprintf ("                  0x1p+28", "%25.0Ra", x);
   check_sprintf ("                   0x0p+0", "%25.0Ra", z);
   /* sign or space, pad with leading zeros */
   check_sprintf (" 0X00F.EDCBA987654321P+24", "% 025RA", x);
-  check_sprintf (" 0X000000000000000008P+25", "% 025.0RA", x);
+  check_sprintf (" 0X000000000000000001P+28", "% 025.0RA", x);
   check_sprintf (" 0X0000000000000000000P+0", "% 025.0RA", z);
   /* sign + or -, left justified */
   check_sprintf ("+0xf.edcba987654321p+24  ", "%+-25Ra", x);
-  check_sprintf ("+0x8p+25                 ", "%+-25.0Ra", x);
+  check_sprintf ("+0x1p+28                 ", "%+-25.0Ra", x);
   check_sprintf ("+0x0p+0                  ", "%+-25.0Ra", z);
   /* decimal point, left justified, precision and rounding parameter */
-  check_vsprintf ("0XF.FP+24 ", "%#-10.*R*A", 1, GMP_RNDN, x);
-  check_vsprintf ("0X8.P+25  ", "%#-10.*R*A", 0, GMP_RNDN, x);
-  check_vsprintf ("0X0.P+0   ", "%#-10.*R*A", 0, GMP_RNDN, z);
+  check_vsprintf ("0XF.FP+24 ", "%#-10.*R*A", 1, MPFR_RNDN, x);
+  check_vsprintf ("0X1.P+28  ", "%#-10.*R*A", 0, MPFR_RNDN, x);
+  check_vsprintf ("0X0.P+0   ", "%#-10.*R*A", 0, MPFR_RNDN, z);
   /* sign or space */
   check_sprintf (" 0xf.eddp+24", "% .3RNa", x);
-  check_sprintf (" 0x8p+25",     "% .0RNa", x);
+  check_sprintf (" 0x1p+28",     "% .0RNa", x);
   /* sign + or -, decimal point, pad with leading zeros */
   check_sprintf ("+0X0F.EP+24", "%0+#11.1RZA", x);
   check_sprintf ("+0X00F.P+24", "%0+#11.0RZA", x);
@@ -448,8 +621,8 @@ hexadecimal (void)
   check_sprintf (" 0XF.EP+24 " , "%- #11.1RDA", x);
   check_sprintf (" 0XF.P+24  " , "%- #11.0RDA", x);
 
-  mpfr_mul_si (x, x, -1, GMP_RNDD);
-  mpfr_mul_si (z, z, -1, GMP_RNDD);
+  mpfr_mul_si (x, x, -1, MPFR_RNDD);
+  mpfr_mul_si (z, z, -1, MPFR_RNDD);
 
   /* sign + or - */
   check_sprintf ("-0xf.ep+24", "%+10.1RUa", x);
@@ -457,15 +630,67 @@ hexadecimal (void)
   check_sprintf ("   -0x0p+0", "%+10.0RUa", z);
 
   /* rounding bit is zero */
-  mpfr_set_str (x, "0xF.7", 16, GMP_RNDN);
+  mpfr_set_str (x, "0xF.7", 16, MPFR_RNDN);
   check_sprintf ("0XFP+0", "%.0RNA", x);
   /* tie case in round to nearest mode */
-  mpfr_set_str (x, "0x0.8800000000000000p+3", 16, GMP_RNDN);
-  check_sprintf ("0x8.p-1", "%#.0RNa", x);
-  mpfr_set_str (x, "-0x0.9800000000000000p+3", 16, GMP_RNDN);
+  mpfr_set_str (x, "0x0.8800000000000000p+3", 16, MPFR_RNDN);
+  check_sprintf ("0x9.p-1", "%#.0RNa", x);
+  mpfr_set_str (x, "-0x0.9800000000000000p+3", 16, MPFR_RNDN);
   check_sprintf ("-0xap-1", "%.0RNa", x);
   /* trailing zeros in fractional part */
   check_sprintf ("-0X4.C0000000000000000000P+0", "%.20RNA", x);
+  /* rounding bit is one and the first non zero bit is far away */
+  mpfr_set_prec (x, 1024);
+  mpfr_set_ui_2exp (x, 29, -1, MPFR_RNDN);
+  mpfr_nextabove (x);
+  check_sprintf ("0XFP+0", "%.0RNA", x);
+
+  /* with more than one limb */
+  mpfr_set_prec (x, 300);
+  mpfr_set_str (x, "0xf.ffffffffffffffffffffffffffffffffffffffffffffffffffff"
+                "fffffffffffffffff", 16, MPFR_RNDN);
+  check_sprintf ("0x1p+4 [300]", "%.0RNa [300]", x);
+  check_sprintf ("0xfp+0 [300]", "%.0RZa [300]", x);
+  check_sprintf ("0x1p+4 [300]", "%.0RYa [300]", x);
+  check_sprintf ("0xfp+0 [300]", "%.0RDa [300]", x);
+  check_sprintf ("0x1p+4 [300]", "%.0RUa [300]", x);
+  check_sprintf ("0x1.0000000000000000000000000000000000000000p+4",
+                 "%.40RNa", x);
+  check_sprintf ("0xf.ffffffffffffffffffffffffffffffffffffffffp+0",
+                 "%.40RZa", x);
+  check_sprintf ("0x1.0000000000000000000000000000000000000000p+4",
+                 "%.40RYa", x);
+  check_sprintf ("0xf.ffffffffffffffffffffffffffffffffffffffffp+0",
+                 "%.40RDa", x);
+  check_sprintf ("0x1.0000000000000000000000000000000000000000p+4",
+                 "%.40RUa", x);
+
+  mpfr_set_str (x, "0xf.7fffffffffffffffffffffffffffffffffffffffffffffffffff"
+                "ffffffffffffffffff", 16, MPFR_RNDN);
+  check_sprintf ("0XFP+0", "%.0RNA", x);
+  check_sprintf ("0XFP+0", "%.0RZA", x);
+  check_sprintf ("0X1P+4", "%.0RYA", x);
+  check_sprintf ("0XFP+0", "%.0RDA", x);
+  check_sprintf ("0X1P+4", "%.0RUA", x);
+  check_sprintf ("0XF.8P+0", "%.1RNA", x);
+  check_sprintf ("0XF.7P+0", "%.1RZA", x);
+  check_sprintf ("0XF.8P+0", "%.1RYA", x);
+  check_sprintf ("0XF.7P+0", "%.1RDA", x);
+  check_sprintf ("0XF.8P+0", "%.1RUA", x);
+
+  /* do not round up to the next power of the base */
+  mpfr_set_str (x, "0xf.fffffffffffffffffffffffffffffffffffffeffffffffffffff"
+                "ffffffffffffffffff", 16, MPFR_RNDN);
+  check_sprintf ("0xf.ffffffffffffffffffffffffffffffffffffff00p+0",
+                 "%.40RNa", x);
+  check_sprintf ("0xf.fffffffffffffffffffffffffffffffffffffeffp+0",
+                 "%.40RZa", x);
+  check_sprintf ("0xf.ffffffffffffffffffffffffffffffffffffff00p+0",
+                 "%.40RYa", x);
+  check_sprintf ("0xf.fffffffffffffffffffffffffffffffffffffeffp+0",
+                 "%.40RDa", x);
+  check_sprintf ("0xf.ffffffffffffffffffffffffffffffffffffff00p+0",
+                 "%.40RUa", x);
 
   mpfr_clears (x, z, (mpfr_ptr) 0);
   return 0;
@@ -489,8 +714,8 @@ binary (void)
   check_sprintf (nan_str, "%Rb", x);
 
   /* regular numbers */
-  mpfr_set_str (x, "1110010101.1001101", 2, GMP_RNDN);
-  mpfr_set_ui (z, 0, GMP_RNDN);
+  mpfr_set_str (x, "1110010101.1001101", 2, MPFR_RNDN);
+  mpfr_set_ui (z, 0, MPFR_RNDN);
 
   /* simplest case: right justified */
   check_sprintf ("    1.1100101011001101p+9", "%25Rb", x);
@@ -512,16 +737,33 @@ binary (void)
   /* pad with leading zero */
   check_sprintf ("00001.1100101011001101p+9", "%025RDb", x);
   /* sign or space, decimal point (unused), left justified */
-  check_sprintf (" 1.1p+9    " , "%- #11.1RDb", x);
-  check_sprintf (" 1.1p+9    " , "%- #11.0RDb", x);
+  check_sprintf (" 1.1p+9    ", "%- #11.1RDb", x);
+  check_sprintf (" 1.p+9     ", "%- #11.0RDb", x);
+  check_sprintf (" 1.p+10    ", "%- #11.0RUb", x);
+  check_sprintf (" 1.p+9     ", "%- #11.0RZb", x);
+  check_sprintf (" 1.p+10    ", "%- #11.0RYb", x);
+  check_sprintf (" 1.p+10    ", "%- #11.0RNb", x);
 
-  mpfr_mul_si (x, x, -1, GMP_RNDD);
-  mpfr_mul_si (z, z, -1, GMP_RNDD);
+  mpfr_mul_si (x, x, -1, MPFR_RNDD);
+  mpfr_mul_si (z, z, -1, MPFR_RNDD);
 
   /* sign + or - */
   check_sprintf ("   -1.1p+9", "%+10.1RUb", x);
   check_sprintf ("   -0.0p+0", "%+10.1RUb", z);
 
+  /* precision 0 */
+  check_sprintf ("-1p+10", "%.0RNb", x);
+  check_sprintf ("-1p+10", "%.0RDb", x);
+  check_sprintf ("-1p+9",  "%.0RUb", x);
+  check_sprintf ("-1p+9",  "%.0RZb", x);
+  check_sprintf ("-1p+10", "%.0RYb", x);
+  /* round to next base power */
+  check_sprintf ("-1.0p+10", "%.1RNb", x);
+  check_sprintf ("-1.0p+10", "%.1RDb", x);
+  check_sprintf ("-1.0p+10", "%.1RYb", x);
+  /* do not round to next base power */
+  check_sprintf ("-1.1p+9", "%.1RUb", x);
+  check_sprintf ("-1.1p+9", "%.1RZb", x);
   /* rounding bit is zero */
   check_sprintf ("-1.11p+9", "%.2RNb", x);
   /* tie case in round to nearest mode */
@@ -539,12 +781,14 @@ mixed (void)
   int n1;
   int n2;
   int i = 121;
+#ifndef NPRINTF_L
   long double d = 1. / 31.;
+#endif
   mpf_t mpf;
   mpq_t mpq;
   mpz_t mpz;
   mpfr_t x;
-  mp_rnd_t rnd;
+  mpfr_rnd_t rnd;
 
   mpf_init (mpf);
   mpf_set_ui (mpf, 40);
@@ -554,8 +798,8 @@ mixed (void)
   mpz_init (mpz);
   mpz_fib_ui (mpz, 64);
   mpfr_init (x);
-  mpfr_set_str (x, "-12345678.875", 10, GMP_RNDN);
-  rnd = GMP_RNDD;
+  mpfr_set_str (x, "-12345678.875", 10, MPFR_RNDN);
+  rnd = MPFR_RNDD;
 
   check_vsprintf ("121%", "%i%%", i);
   check_vsprintf ("121% -1.2345678875E+07", "%i%% %RNE", i, x);
@@ -563,7 +807,7 @@ mixed (void)
   check_vsprintf ("10610209857723, -1.2345678875e+07", "%Zi, %R*e", mpz, rnd,
                   x);
   check_vsprintf ("-12345678.9, 121", "%.1Rf, %i", x, i);
-  check_vsprintf ("-12345678, 1e240/45b352", "%.0R*f, %Qx", GMP_RNDZ, x, mpq);
+  check_vsprintf ("-12345678, 1e240/45b352", "%.0R*f, %Qx", MPFR_RNDZ, x, mpq);
   n1 = check_vsprintf ("121, -12345678.875000000000, 1.290323", "%i, %.*Rf, %Ff%n",
                        i, 12, x, mpf, &n2);
   if (n1 != n2)
@@ -586,41 +830,44 @@ mixed (void)
   return 0;
 }
 
-/* Check with locale "da_DK"
-   decimal point is ',' and thousands separator is '.' */
+/* Check with locale "da_DK". On most platforms, decimal point is ','
+   and thousands separator is '.'; the test is not performed if this
+   is not the case or if the locale doesn't exist. */
 static int
 locale_da_DK (void)
 {
   mpfr_prec_t p = 128;
   mpfr_t x;
 
-  if (setlocale (LC_ALL, "da_DK") == 0)
+  if (setlocale (LC_ALL, "da_DK") == 0 ||
+      localeconv()->decimal_point[0] != ',' ||
+      localeconv()->thousands_sep[0] != '.')
     return 0;
 
   mpfr_init2 (x, p);
 
   /* positive numbers */
-  mpfr_set_str (x, "18993474.61279296875", 10, GMP_RNDN);
+  mpfr_set_str (x, "18993474.61279296875", 10, MPFR_RNDN);
 
   /* simplest case right justified with thousands separator */
   check_sprintf ("      1,899347461279296875e+07", "%'30Re", x);
   check_sprintf ("                   1,89935e+07", "%'30Rg", x);
   check_sprintf ("        18.993.474,61279296875", "%'30.19Rg", x);
-  check_sprintf ("        18.993.474,61279296875", "%'30Rf", x);
+  check_sprintf ("             18.993.474,612793", "%'30Rf", x);
 
   /* sign or space, pad, thousands separator with leading zeros */
   check_sprintf (" 000001,899347461279296875E+07", "%' 030RE", x);
   check_sprintf (" 0000000000000000001,89935E+07", "%' 030RG", x);
   check_sprintf (" 000000018.993.474,61279296875", "%' 030.19RG", x);
-  check_sprintf (" 000000018.993.474,61279296875", "%' 030RF", x);
+  check_sprintf (" 00000000000018.993.474,612793", "%' 030RF", x);
 
-  mpfr_set_ui (x, 50, GMP_RNDN);
-  mpfr_exp10 (x, x, GMP_RNDN);
-  check_sprintf ("100000000000000000000000000000000000000000000000000", "%Rf",
+  mpfr_set_ui (x, 50, MPFR_RNDN);
+  mpfr_exp10 (x, x, MPFR_RNDN);
+  check_sprintf ("100000000000000000000000000000000000000000000000000", "%.0Rf",
                  x);
   check_sprintf
     ("100.000.000.000.000.000.000.000.000.000.000.000.000.000.000.000.000,",
-     "%'#Rf", x);
+     "%'#.0Rf", x);
   check_sprintf
     ("100.000.000.000.000.000.000.000.000.000.000.000.000.000.000.000.000,0000",
      "%'.4Rf", x);
@@ -697,8 +944,8 @@ random_double (void)
       if (randlimb () % 2 == 0)
         y = -y;
 
-      mpfr_set_d (x, y, GMP_RNDN);
-      if (y != mpfr_get_d (x, GMP_RNDN))
+      mpfr_set_d (x, y, MPFR_RNDN);
+      if (y != mpfr_get_d (x, MPFR_RNDN))
         /* conversion error: skip this one */
         continue;
 
@@ -787,7 +1034,7 @@ bug20080610 (void)
   mpfr_init2 (x, MPFR_LDBL_MANT_DIG);
 
   y = -9.95645044213728791504536275169812142849e-01;
-  mpfr_set_d (x, y, GMP_RNDN);
+  mpfr_set_d (x, y, MPFR_RNDN);
 
   xi = mpfr_asprintf (&xs, "%- #0.*Re", 1, x);
   yi = mpfr_asprintf (&ys, "%- #0.*e", 1, y);
@@ -822,7 +1069,7 @@ bug20081214 (void)
   mpfr_init2 (x, MPFR_LDBL_MANT_DIG);
 
   y = -9.90597761233942053494e-01;
-  mpfr_set_d (x, y, GMP_RNDN);
+  mpfr_set_d (x, y, MPFR_RNDN);
 
   xi = mpfr_asprintf (&xs, "%- #0.*RG", 1, x);
   yi = mpfr_asprintf (&ys, "%- #0.*G", 1, y);
@@ -843,6 +1090,94 @@ bug20081214 (void)
   mpfr_clear (x);
 }
 
+static void
+bug20111102 (void)
+{
+  mpfr_t t;
+  char s[100];
+
+  mpfr_init2 (t, 84);
+  mpfr_set_str (t, "999.99999999999999999999", 10, MPFR_RNDN);
+  mpfr_sprintf (s, "%.20RNg", t);
+  if (strcmp (s, "1000") != 0)
+    {
+      printf ("Error in bug20111102, expected 1000, got %s\n", s);
+      exit (1);
+    }
+  mpfr_clear (t);
+}
+
+/* In particular, the following test makes sure that the rounding
+ * for %Ra and %Rb is not done on the MPFR number itself (as it
+ * would overflow). Note: it has been reported on comp.std.c that
+ * some C libraries behave differently on %a, but this is a bug.
+ */
+static void
+check_emax_aux (mpfr_exp_t e)
+{
+  mpfr_t x;
+  char *s1, s2[256];
+  int i;
+  mpfr_exp_t emax;
+
+  MPFR_ASSERTN (e <= LONG_MAX);
+  emax = mpfr_get_emax ();
+  set_emax (e);
+
+  mpfr_init2 (x, 16);
+
+  mpfr_set_inf (x, 1);
+  mpfr_nextbelow (x);
+
+  i = mpfr_asprintf (&s1, "%Ra %.2Ra", x, x);
+  MPFR_ASSERTN (i > 0);
+
+  mpfr_snprintf (s2, 256, "0x7.fff8p+%ld 0x8.00p+%ld", e-3, e-3);
+
+  if (strcmp (s1, s2) != 0)
+    {
+      printf ("Error in check_emax_aux for emax = ");
+      if (e > LONG_MAX)
+        printf ("(>LONG_MAX)\n");
+      else
+        printf ("%ld\n", (long) e);
+      printf ("Expected %s\n", s2);
+      printf ("Got      %s\n", s1);
+      exit (1);
+    }
+
+  mpfr_free_str (s1);
+
+  i = mpfr_asprintf (&s1, "%Rb %.2Rb", x, x);
+  MPFR_ASSERTN (i > 0);
+
+  mpfr_snprintf (s2, 256, "1.111111111111111p+%ld 1.00p+%ld", e-1, e);
+
+  if (strcmp (s1, s2) != 0)
+    {
+      printf ("Error in check_emax_aux for emax = ");
+      if (e > LONG_MAX)
+        printf ("(>LONG_MAX)\n");
+      else
+        printf ("%ld\n", (long) e);
+      printf ("Expected %s\n", s2);
+      printf ("Got      %s\n", s1);
+      exit (1);
+    }
+
+  mpfr_free_str (s1);
+
+  mpfr_clear (x);
+  set_emax (emax);
+}
+
+static void
+check_emax (void)
+{
+  check_emax_aux (15);
+  check_emax_aux (MPFR_EMAX_MAX);
+}
+
 int
 main (int argc, char **argv)
 {
@@ -855,11 +1190,13 @@ main (int argc, char **argv)
   locale = setlocale (LC_ALL, "C");
 #endif
 
+  bug20111102 ();
   native_types ();
   hexadecimal ();
   binary ();
   decimal ();
   mixed ();
+  check_emax ();
 
 #if defined(HAVE_LOCALE_H) && defined(HAVE_SETLOCALE)
   locale_da_DK ();
@@ -896,7 +1233,7 @@ int
 main (void)
 {
   /* We have nothing to test. */
-  return 0;
+  return 77;
 }
 
 #endif  /* HAVE_STDARG */
